@@ -26,6 +26,7 @@
 #include <config.h>
 #include "activation.h"
 #include "activation-exit-codes.h"
+#include "config-parser.h"
 #include "desktop-file.h"
 #include "dispatch.h"
 #include "services.h"
@@ -53,7 +54,7 @@ struct BusActivation
                               * i.e. number of pending activation requests, not pending
                               * activations per se
                               */
-  DBusHashTable *directories;
+  DBusList *directories;
   DBusHashTable *environment;
 };
 
@@ -61,6 +62,7 @@ typedef struct
 {
   int refcount;
   char *dir_c;
+  BusServiceDirFlags flags;
   DBusHashTable *entries;
 } BusServiceDirectory;
 
@@ -400,8 +402,69 @@ update_desktop_file_entry (BusActivation       *activation,
 
   if (entry == NULL) /* New file */
     {
+      DBusString expected_name;
+
+      if (!_dbus_string_init (&expected_name))
+        {
+          BUS_SET_OOM (error);
+          goto out;
+        }
+
+      if (!_dbus_string_append (&expected_name, name) ||
+          !_dbus_string_append (&expected_name, ".service"))
+        {
+          _dbus_string_free (&expected_name);
+          BUS_SET_OOM (error);
+          goto out;
+        }
+
+      if (_dbus_string_equal (&expected_name, filename))
+        {
+          _dbus_verbose ("Name of \"%s\" is as expected\n",
+                         _dbus_string_get_const_data (&file_path));
+        }
+      else if (s_dir->flags & BUS_SERVICE_DIR_FLAGS_STRICT_NAMING)
+        {
+          bus_context_log_and_set_error (activation->context,
+                                         DBUS_SYSTEM_LOG_WARNING, error,
+                                         DBUS_ERROR_FAILED,
+                                         "Service file \"%s\" should have "
+                                         "been named \"%s\": not loading it",
+                                         _dbus_string_get_const_data (&file_path),
+                                         _dbus_string_get_const_data (&expected_name));
+          _dbus_string_free (&expected_name);
+          goto out;
+        }
+      else if (bus_context_get_servicehelper (activation->context) != NULL)
+        {
+          bus_context_log (activation->context, DBUS_SYSTEM_LOG_WARNING,
+                           "Service file \"%s\" should have been named \"%s\""
+                           "and will not work with system bus activation",
+                           _dbus_string_get_const_data (&file_path),
+                           _dbus_string_get_const_data (&expected_name));
+          /* We don't actually error out here, because *technically* it could
+           * still work on systemd systems, where we tell systemd to start the
+           * SystemdService instead of launching dbus-daemon-launch-helper
+           * ourselves. But maybe we should:
+           * https://bugs.freedesktop.org/show_bug.cgi?id=99874 */
+        }
+      else
+        {
+          /* We could maybe log mismatched names for session services in
+           * a user-visible way too, but not until
+           * https://lintian.debian.org/tags/dbus-session-service-wrong-name.html
+           * is a bit shorter.
+           * https://bugs.freedesktop.org/show_bug.cgi?id=99873 */
+          _dbus_verbose ("Name of \"%s\" should canonically be \"%s\"\n",
+                         _dbus_string_get_const_data (&file_path),
+                         _dbus_string_get_const_data (&expected_name));
+        }
+
+      _dbus_string_free (&expected_name);
+
       /* FIXME we need a better-defined algorithm for which service file to
        * pick than "whichever one is first in the directory listing"
+       * See also https://bugs.freedesktop.org/show_bug.cgi?id=99874
        */
       if (_dbus_hash_table_lookup_string (activation->entries, name))
         {
@@ -814,23 +877,19 @@ bus_activation_reload (BusActivation     *activation,
       goto failed;
     }
 
-  if (activation->directories != NULL)
-    _dbus_hash_table_unref (activation->directories);
-  activation->directories = _dbus_hash_table_new (DBUS_HASH_STRING, NULL,
-                                                  (DBusFreeFunction)bus_service_directory_unref);
-
-  if (activation->directories == NULL)
-    {
-      BUS_SET_OOM (error);
-      goto failed;
-    }
+  _dbus_list_foreach (&activation->directories,
+                      (DBusForeachFunction) bus_service_directory_unref, NULL);
+  _dbus_list_clear (&activation->directories);
 
   link = _dbus_list_get_first_link (directories);
   while (link != NULL)
     {
+      BusConfigServiceDir *config = link->data;
       BusServiceDirectory *s_dir;
 
-      dir = _dbus_strdup ((const char *) link->data);
+      _dbus_assert (config->path != NULL);
+
+      dir = _dbus_strdup (config->path);
       if (!dir)
         {
           BUS_SET_OOM (error);
@@ -847,6 +906,7 @@ bus_activation_reload (BusActivation     *activation,
 
       s_dir->refcount = 1;
       s_dir->dir_c = dir;
+      s_dir->flags = config->flags;
 
       s_dir->entries = _dbus_hash_table_new (DBUS_HASH_STRING, NULL,
                                              (DBusFreeFunction)bus_activation_entry_unref);
@@ -858,7 +918,7 @@ bus_activation_reload (BusActivation     *activation,
           goto failed;
         }
 
-      if (!_dbus_hash_table_insert_string (activation->directories, s_dir->dir_c, s_dir))
+      if (!_dbus_list_append (&activation->directories, s_dir))
         {
           bus_service_directory_unref (s_dir);
           BUS_SET_OOM (error);
@@ -965,8 +1025,11 @@ bus_activation_unref (BusActivation *activation)
     _dbus_hash_table_unref (activation->entries);
   if (activation->pending_activations)
     _dbus_hash_table_unref (activation->pending_activations);
-  if (activation->directories)
-    _dbus_hash_table_unref (activation->directories);
+
+  _dbus_list_foreach (&activation->directories,
+                      (DBusForeachFunction) bus_service_directory_unref, NULL);
+  _dbus_list_clear (&activation->directories);
+
   if (activation->environment)
     _dbus_hash_table_unref (activation->environment);
 
@@ -1537,15 +1600,14 @@ add_cancel_pending_to_transaction (BusTransaction       *transaction,
 static dbus_bool_t
 update_service_cache (BusActivation *activation, DBusError *error)
 {
-  DBusHashIter iter;
+  DBusList *iter;
 
-  _dbus_hash_iter_init (activation->directories, &iter);
-  while (_dbus_hash_iter_next (&iter))
+  for (iter = _dbus_list_get_first_link (&activation->directories);
+       iter != NULL;
+       iter = _dbus_list_get_next_link (&activation->directories, iter))
     {
       DBusError tmp_error;
-      BusServiceDirectory *s_dir;
-
-      s_dir = _dbus_hash_iter_get_value (&iter);
+      BusServiceDirectory *s_dir = iter->data;
 
       dbus_error_init (&tmp_error);
       if (!update_directory (activation, s_dir, &tmp_error))
@@ -2312,9 +2374,9 @@ bus_activation_entry_get_assumed_apparmor_label (BusActivationEntry *entry)
 
 #include <stdio.h>
 
-#define SERVICE_NAME_1 "MyService1"
-#define SERVICE_NAME_2 "MyService2"
-#define SERVICE_NAME_3 "MyService3"
+#define SERVICE_NAME_1 "com.example.MyService1"
+#define SERVICE_NAME_2 "org.example.MyService2"
+#define SERVICE_NAME_3 "net.example.MyService3"
 
 #define SERVICE_FILE_1 "service-1.service"
 #define SERVICE_FILE_2 "service-2.service"
@@ -2441,21 +2503,8 @@ out:
 static dbus_bool_t
 init_service_reload_test (DBusString *dir)
 {
-  DBusStat stat_buf;
-
-  if (!_dbus_stat (dir, &stat_buf, NULL))
-    {
-      if (!_dbus_create_directory (dir, NULL))
-        return FALSE;
-    }
-  else
-    {
-      if (!test_remove_directory (dir))
-        return FALSE;
-
-      if (!_dbus_create_directory (dir, NULL))
-        return FALSE;
-    }
+  if (!_dbus_create_directory (dir, NULL))
+    return FALSE;
 
   /* Create one initial file */
   if (!test_create_service_file (dir, SERVICE_FILE_1, SERVICE_NAME_1, "exec-1"))
@@ -2534,9 +2583,13 @@ do_test (const char *description, dbus_bool_t oom_test, CheckData *data)
 }
 
 static dbus_bool_t
-do_service_reload_test (DBusString *dir, dbus_bool_t oom_test)
+do_service_reload_test (const DBusString *test_data_dir,
+                        DBusString       *dir,
+                        dbus_bool_t       oom_test)
 {
   BusActivation *activation;
+  BusConfigServiceDir config;
+  BusContext    *context;
   DBusString     address;
   DBusList      *directories;
   CheckData      d;
@@ -2544,10 +2597,18 @@ do_service_reload_test (DBusString *dir, dbus_bool_t oom_test)
   directories = NULL;
   _dbus_string_init_const (&address, "");
 
-  if (!_dbus_list_append (&directories, _dbus_string_get_data (dir)))
+  config.path = _dbus_string_get_data (dir);
+  config.flags = BUS_SERVICE_DIR_FLAGS_NONE;
+
+  if (!_dbus_list_append (&directories, &config))
     return FALSE;
 
-  activation = bus_activation_new (NULL, &address, &directories, NULL);
+  context = bus_context_new_test (test_data_dir,
+                                  "valid-config-files/debug-allow-all.conf");
+  if (context == NULL)
+    return FALSE;
+
+  activation = bus_activation_new (context, &address, &directories, NULL);
   if (!activation)
     return FALSE;
 
@@ -2608,6 +2669,7 @@ do_service_reload_test (DBusString *dir, dbus_bool_t oom_test)
 
   bus_activation_unref (activation);
   _dbus_list_clear (&directories);
+  bus_context_unref (context);
 
   return TRUE;
 }
@@ -2638,16 +2700,19 @@ bus_activation_service_reload_test (const DBusString *test_data_dir)
   if (!init_service_reload_test (&directory))
     _dbus_assert_not_reached ("could not initiate service reload test");
 
-  if (!do_service_reload_test (&directory, FALSE))
+  if (!do_service_reload_test (test_data_dir, &directory, FALSE))
     {
       /* Do nothing? */
     }
 
+  if (!cleanup_service_reload_test (&directory))
+    goto out;
+
   /* Do OOM tests */
   if (!init_service_reload_test (&directory))
     _dbus_assert_not_reached ("could not initiate service reload test");
 
-  if (!do_service_reload_test (&directory, TRUE))
+  if (!do_service_reload_test (test_data_dir, &directory, TRUE))
     {
       /* Do nothing? */
     }
