@@ -529,7 +529,6 @@ sha1_handle_first_client_response (DBusAuth         *auth,
   DBusString tmp2;
   dbus_bool_t retval = FALSE;
   DBusError error = DBUS_ERROR_INIT;
-  DBusCredentials *myself = NULL;
 
   _dbus_string_set_length (&auth->challenge, 0);
   
@@ -566,34 +565,6 @@ sha1_handle_first_client_response (DBusAuth         *auth,
       return FALSE;
     }
 
-  myself = _dbus_credentials_new_from_current_process ();
-
-  if (myself == NULL)
-    goto out;
-
-  if (!_dbus_credentials_same_user (myself, auth->desired_identity))
-    {
-      /*
-       * DBUS_COOKIE_SHA1 is not suitable for authenticating that the
-       * client is anyone other than the user owning the process
-       * containing the DBusServer: we probably aren't allowed to write
-       * to other users' home directories. Even if we can (for example
-       * uid 0 on traditional Unix or CAP_DAC_OVERRIDE on Linux), we
-       * must not, because the other user controls their home directory,
-       * and could carry out symlink attacks to make us read from or
-       * write to unintended locations. It's difficult to avoid symlink
-       * attacks in a portable way, so we just don't try. This isn't a
-       * regression, because DBUS_COOKIE_SHA1 never worked for other
-       * users anyway.
-       */
-      _dbus_verbose ("%s: client tried to authenticate as \"%s\", "
-                     "but that doesn't match this process",
-                     DBUS_AUTH_NAME (auth),
-                     _dbus_string_get_const_data (data));
-      retval = send_rejected (auth);
-      goto out;
-    }
-
   /* we cache the keyring for speed, so here we drop it if it's the
    * wrong one. FIXME caching the keyring here is useless since we use
    * a different DBusAuth for every connection.
@@ -708,9 +679,6 @@ sha1_handle_first_client_response (DBusAuth         *auth,
   _dbus_string_zero (&tmp2);
   _dbus_string_free (&tmp2);
 
-  if (myself != NULL)
-    _dbus_credentials_unref (myself);
-
   return retval;
 }
 
@@ -1517,9 +1485,14 @@ send_rejected (DBusAuth *auth)
                             "REJECTED"))
     goto nomem;
 
-  i = 0;
-  while (all_mechanisms[i].mechanism != NULL)
+  for (i = 0; all_mechanisms[i].mechanism != NULL; i++)
     {
+      /* skip mechanisms that aren't allowed */
+      if (auth->allowed_mechs != NULL &&
+          !_dbus_string_array_contains ((const char**)auth->allowed_mechs,
+                                        all_mechanisms[i].mechanism))
+        continue;
+
       if (!_dbus_string_append (&command,
                                 " "))
         goto nomem;
@@ -1527,8 +1500,6 @@ send_rejected (DBusAuth *auth)
       if (!_dbus_string_append (&command,
                                 all_mechanisms[i].mechanism))
         goto nomem;
-      
-      ++i;
     }
   
   if (!_dbus_string_append (&command, "\r\n"))
@@ -1637,8 +1608,13 @@ process_ok(DBusAuth *auth,
   _dbus_verbose ("Got GUID '%s' from the server\n",
                  _dbus_string_get_const_data (& DBUS_AUTH_CLIENT (auth)->guid_from_server));
 
-  if (auth->unix_fd_possible)
-    return send_negotiate_unix_fd(auth);
+  if (auth->unix_fd_possible) {
+     if (!send_negotiate_unix_fd(auth)) {
+         _dbus_string_set_length (& DBUS_AUTH_CLIENT (auth)->guid_from_server, 0);
+         return FALSE;
+     }
+     return TRUE;
+  }
 
   _dbus_verbose("Not negotiating unix fd passing, since not possible\n");
   return send_begin (auth);
@@ -2869,6 +2845,45 @@ _dbus_auth_get_unix_fd_negotiated(DBusAuth *auth)
   return auth->unix_fd_negotiated;
 }
 
+/**
+ * Queries whether the given auth mechanism is supported.
+ *
+ * @param auth the auth mechanism to query for
+ * @returns #TRUE when auth mechanism is supported
+ */
+dbus_bool_t
+_dbus_auth_is_supported_mechanism (DBusString *name)
+{
+  _dbus_assert (name != NULL);
+
+  return find_mech (name, NULL) != NULL;
+}
+
+/**
+ * Return a human-readable string containing all supported auth mechanisms.
+ *
+ * @param string to hold the supported auth mechanisms
+ * @returns #FALSE on oom
+ */
+dbus_bool_t
+_dbus_auth_dump_supported_mechanisms (DBusString *buffer)
+{
+  unsigned int i;
+  _dbus_assert (buffer != NULL);
+
+  for (i = 0; all_mechanisms[i].mechanism != NULL; i++)
+    {
+      if (i > 0)
+        {
+          if (!_dbus_string_append (buffer, ", "))
+            return FALSE;
+        }
+      if (!_dbus_string_append (buffer, all_mechanisms[i].mechanism))
+        return FALSE;
+    }
+  return TRUE;
+}
+
 /** @} */
 
 /* tests in dbus-auth-util.c */
