@@ -66,7 +66,9 @@ run_validity_tests (const ValidityTest *tests,
 
 static const ValidityTest signature_tests[] = {
   { "", DBUS_VALID },
+  { "sss", DBUS_VALID },
   { "i", DBUS_VALID },
+  { "b", DBUS_VALID },
   { "ai", DBUS_VALID },
   { "(i)", DBUS_VALID },
   { "w", DBUS_INVALID_UNKNOWN_TYPECODE },
@@ -95,9 +97,15 @@ static const ValidityTest signature_tests[] = {
   { "a{ia}", DBUS_INVALID_MISSING_ARRAY_ELEMENT_TYPE },
   { "a{}", DBUS_INVALID_DICT_ENTRY_HAS_NO_FIELDS },
   { "a{aii}", DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE },
-  /* { "a{i}", DBUS_INVALID_DICT_ENTRY_HAS_ONLY_ONE_FIELD }, */
-  /* { "{is}", DBUS_INVALID_DICT_ENTRY_NOT_INSIDE_ARRAY }, */
-  /* { "a{isi}", DBUS_INVALID_DICT_ENTRY_HAS_TOO_MANY_FIELDS }, */
+  { " ", DBUS_INVALID_UNKNOWN_TYPECODE },
+  { "not a valid signature", DBUS_INVALID_UNKNOWN_TYPECODE },
+  { "123", DBUS_INVALID_UNKNOWN_TYPECODE },
+  { ".", DBUS_INVALID_UNKNOWN_TYPECODE },
+  /* https://bugs.freedesktop.org/show_bug.cgi?id=17803 */
+  { "a{(ii)i}", DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE },
+  { "a{i}", DBUS_INVALID_DICT_ENTRY_HAS_ONLY_ONE_FIELD },
+  { "{is}", DBUS_INVALID_DICT_ENTRY_NOT_INSIDE_ARRAY },
+  { "a{isi}", DBUS_INVALID_DICT_ENTRY_HAS_TOO_MANY_FIELDS },
 };
 
 dbus_bool_t
@@ -213,22 +221,6 @@ _dbus_marshal_validate_test (void)
     "foo bar"
   };
 
-  const char *valid_signatures[] = {
-    "",
-    "sss",
-    "i",
-    "b"
-  };
-
-  const char *invalid_signatures[] = {
-    " ",
-    "not a valid signature",
-    "123",
-    ".",
-    "(",
-    "a{(ii)i}" /* https://bugs.freedesktop.org/show_bug.cgi?id=17803 */
-  };
-
   /* Signature with reason */
 
   run_validity_tests (signature_tests, _DBUS_N_ELEMENTS (signature_tests),
@@ -430,37 +422,6 @@ _dbus_marshal_validate_test (void)
       ++i;
     }
 
-  /* Signature validation */
-  i = 0;
-  while (i < (int) _DBUS_N_ELEMENTS (valid_signatures))
-    {
-      _dbus_string_init_const (&str, valid_signatures[i]);
-
-      if (!_dbus_validate_signature (&str, 0,
-                                     _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Signature \"%s\" should have been valid", valid_signatures[i]);
-          _dbus_assert_not_reached ("invalid signature");
-        }
-
-      ++i;
-    }
-
-  i = 0;
-  while (i < (int) _DBUS_N_ELEMENTS (invalid_signatures))
-    {
-      _dbus_string_init_const (&str, invalid_signatures[i]);
-
-      if (_dbus_validate_signature (&str, 0,
-                                    _dbus_string_get_length (&str)))
-        {
-          _dbus_warn ("Signature \"%s\" should have been invalid", invalid_signatures[i]);
-          _dbus_assert_not_reached ("valid signature");
-        }
-
-      ++i;
-    }
-
   /* Validate claimed length longer than real length */
   _dbus_string_init_const (&str, "abc.efg");
   if (_dbus_validate_bus_name (&str, 0, 8))
@@ -474,10 +435,6 @@ _dbus_marshal_validate_test (void)
   if (_dbus_validate_member (&str, 0, 4))
     _dbus_assert_not_reached ("validated too-long string");
 
-  _dbus_string_init_const (&str, "sss");
-  if (_dbus_validate_signature (&str, 0, 4))
-    _dbus_assert_not_reached ("validated too-long signature");
-
   /* Validate string exceeding max name length */
   if (!_dbus_string_init (&str))
     _dbus_assert_not_reached ("no memory");
