@@ -898,7 +898,9 @@ _dbus_validity_to_error_message (DBusValidity validity)
     case DBUS_INVALID_DICT_ENTRY_HAS_TOO_MANY_FIELDS:              return "Dict entry has too many fields";
     case DBUS_INVALID_DICT_ENTRY_NOT_INSIDE_ARRAY:                 return "Dict entry not inside array";
     case DBUS_INVALID_DICT_KEY_MUST_BE_BASIC_TYPE:                 return "Dict key must be basic type";
+    case DBUS_INVALID_MISSING_UNIX_FDS:                            return "Unix file descriptor missing";
     case DBUS_INVALID_NESTED_TOO_DEEPLY:                           return "Variants cannot be used to create a hugely recursive tree of values";
+    case DBUS_VALIDITY_LAST:
     default:
       return "Invalid";
     }
@@ -1212,33 +1214,6 @@ _dbus_validate_bus_namespace (const DBusString  *str,
   return _dbus_validate_bus_name_full (str, start, len, TRUE);
 }
 
-/**
- * Checks that the given range of the string is a valid message type
- * signature in the D-Bus protocol.
- *
- * @todo this is inconsistent with most of DBusString in that
- * it allows a start,len range that extends past the string end.
- *
- * @param str the string
- * @param start first byte index to check
- * @param len number of bytes to check
- * @returns #TRUE if the byte range exists and is a valid signature
- */
-dbus_bool_t
-_dbus_validate_signature (const DBusString  *str,
-                          int                start,
-                          int                len)
-{
-  _dbus_assert (start >= 0);
-  _dbus_assert (start <= _dbus_string_get_length (str));
-  _dbus_assert (len >= 0);
-
-  if (len > _dbus_string_get_length (str) - start)
-    return FALSE;
-
-  return _dbus_validate_signature_with_reason (str, start, len) == DBUS_VALID;
-}
-
 /** define _dbus_check_is_valid_path() */
 DEFINE_DBUS_NAME_CHECK(path)
 /** define _dbus_check_is_valid_interface() */
@@ -1249,8 +1224,6 @@ DEFINE_DBUS_NAME_CHECK(member)
 DEFINE_DBUS_NAME_CHECK(error_name)
 /** define _dbus_check_is_valid_bus_name() */
 DEFINE_DBUS_NAME_CHECK(bus_name)
-/** define _dbus_check_is_valid_signature() */
-DEFINE_DBUS_NAME_CHECK(signature)
 /** define _dbus_check_is_valid_utf8() */
 DEFINE_DBUS_NAME_CHECK(utf8)
 
