@@ -115,13 +115,13 @@ _dbus_string_find_byte_backward (const DBusString  *str,
 
 /** @} */
 
-#ifdef DBUS_BUILD_TESTS
+#ifdef DBUS_ENABLE_EMBEDDED_TESTS
 #include "dbus-test.h"
 #include <stdio.h>
 
 static void
-test_hex_roundtrip (const unsigned char *data,
-                    int                  len)
+test_hex_roundtrip (const char *data,
+                    int         len)
 {
   DBusString orig;
   DBusString encoded;
@@ -170,8 +170,8 @@ test_hex_roundtrip (const unsigned char *data,
   _dbus_string_free (&decoded);  
 }
 
-typedef void (* TestRoundtripFunc) (const unsigned char *data,
-                                    int                  len);
+typedef void (* TestRoundtripFunc) (const char *data,
+                                    int         len);
 static void
 test_roundtrips (TestRoundtripFunc func)
 {
@@ -200,13 +200,13 @@ test_roundtrips (TestRoundtripFunc func)
     i = 0;
     while (i < _DBUS_N_ELEMENTS (buf))
       {
-        buf[i] = i;
+        buf[i] = (i & 0xff);
         ++i;
       }
     i = 0;
     while (i < _DBUS_N_ELEMENTS (buf))
       {
-        (* func) (buf, i);
+        (* func) ((const char *) buf, i);
         ++i;
       }
   }
@@ -312,12 +312,12 @@ _dbus_string_test (void)
   while (i < 10)
     {
       if (!_dbus_string_append (&str, "a"))
-        _dbus_assert_not_reached ("failed to append string to string\n");
+        _dbus_assert_not_reached ("failed to append string to string");
 
       _dbus_assert (_dbus_string_get_length (&str) == i * 2 + 1);
 
       if (!_dbus_string_append_byte (&str, 'b'))
-        _dbus_assert_not_reached ("failed to append byte to string\n");
+        _dbus_assert_not_reached ("failed to append byte to string");
 
       _dbus_assert (_dbus_string_get_length (&str) == i * 2 + 2);
                     
@@ -935,4 +935,4 @@ _dbus_string_test (void)
   return TRUE;
 }
 
-#endif /* DBUS_BUILD_TESTS */
+#endif /* DBUS_ENABLE_EMBEDDED_TESTS */
