@@ -28,29 +28,72 @@
 #include <dbus/dbus-string.h>
 #include <dbus/dbus-marshal-validate.h>
 
+/* Only things that are in libdbus-1.la and used from libdbus-internal.la
+ * need to have DBUS_PRIVATE_EXPORT. If you get
+ *
+ * warning: 'foo' redeclared without dllimport attribute: previous
+ * dllimport ignored [-Wattributes]
+ *
+ * then you have added too many.
+ */
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_hash_test              (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_list_test              (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_marshal_test           (void);
+
 dbus_bool_t _dbus_marshal_recursive_test (void);
 dbus_bool_t _dbus_marshal_byteswap_test  (void);
 dbus_bool_t _dbus_marshal_validate_test  (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_misc_test              (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_signature_test         (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_mem_pool_test          (void);
+
 dbus_bool_t _dbus_string_test            (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_address_test           (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_server_test            (void);
+
 dbus_bool_t _dbus_message_test           (const char *test_data_dir);
 dbus_bool_t _dbus_auth_test              (const char *test_data_dir);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_sha_test               (const char *test_data_dir);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_keyring_test           (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_data_slot_test         (void);
+
 dbus_bool_t _dbus_sysdeps_test           (void);
 dbus_bool_t _dbus_spawn_test             (const char *test_data_dir);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_userdb_test            (const char *test_data_dir);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_transport_unix_test    (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_memory_test            (void);
+
+DBUS_PRIVATE_EXPORT
 dbus_bool_t _dbus_object_tree_test       (void);
+
 dbus_bool_t _dbus_credentials_test       (const char *test_data_dir);
 
 void        dbus_internal_do_not_use_run_tests         (const char          *test_data_dir,
