@@ -125,7 +125,7 @@ echo_filter (DBusConnection *connection,
   if (!dbus_connection_send (connection, reply, NULL))
     g_error ("OOM");
 
-  dbus_message_unref (reply);
+  dbus_clear_message (&reply);
 
   return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -158,6 +158,9 @@ setup (Fixture *f,
 {
   const Config *config = context;
 
+  /* Some tests are fairly slow, so make the test timeout per-test */
+  test_timeout_reset ();
+
   f->ctx = test_main_context_get ();
   f->ge = NULL;
   dbus_error_init (&f->e);
@@ -287,8 +290,8 @@ test_echo (Fixture *f,
             NULL))
         g_error ("OOM");
 
-      dbus_pending_call_unref (pc);
-      dbus_message_unref (m);
+      dbus_clear_pending_call (&pc);
+      dbus_clear_message (&m);
     }
 
   while (received < count)
@@ -341,8 +344,8 @@ test_no_reply (Fixture *f,
         &reply, NULL))
     g_error ("OOM");
 
-  dbus_pending_call_unref (pc);
-  dbus_message_unref (m);
+  dbus_clear_pending_call (&pc);
+  dbus_clear_message (&m);
 
   if (mode == DISCONNECT)
     {
@@ -351,8 +354,7 @@ test_no_reply (Fixture *f,
 
       dbus_connection_remove_filter (f->right_conn, echo_filter, f);
       dbus_connection_close (f->right_conn);
-      dbus_connection_unref (f->right_conn);
-      f->right_conn = NULL;
+      dbus_clear_connection (&f->right_conn);
     }
 
   while (reply == NULL)
@@ -373,7 +375,7 @@ test_no_reply (Fixture *f,
     g_assert_cmpstr (f->e.message, ==,
         "Message did not receive a reply (timeout by message bus)");
 
-  dbus_message_unref (reply);
+  dbus_clear_message (&reply);
 }
 
 static void
@@ -408,8 +410,7 @@ test_creds (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -534,8 +535,8 @@ test_creds (Fixture *f,
   g_assert (seen & SEEN_WINDOWS_SID);
 #endif
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -562,8 +563,7 @@ test_processid (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -607,8 +607,8 @@ test_processid (Fixture *f,
       g_error ("Unexpected error: %s: %s", error.name, error.message);
     }
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -637,8 +637,7 @@ test_canonical_path_uae (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -653,9 +652,8 @@ test_canonical_path_uae (Fixture *f,
   g_assert_cmpint (dbus_message_get_type (m), ==,
       DBUS_MESSAGE_TYPE_METHOD_RETURN);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
-  pc = NULL;
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 
   /* Now try with the wrong object path */
   m = dbus_message_new_method_call (DBUS_SERVICE_DBUS,
@@ -677,8 +675,7 @@ test_canonical_path_uae (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -695,8 +692,8 @@ test_canonical_path_uae (Fixture *f,
       DBUS_ERROR_ACCESS_DENIED);
   g_assert_cmpstr (dbus_message_get_signature (m), ==, "s");
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -733,13 +730,11 @@ test_max_connections (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_LIMITS_EXCEEDED);
 
   if (failing_conn != NULL)
-    {
-      dbus_connection_close (failing_conn);
-      dbus_connection_unref (failing_conn);
-    }
+    dbus_connection_close (failing_conn);
 
+  dbus_clear_connection (&failing_conn);
   dbus_connection_close (third_conn);
-  dbus_connection_unref (third_conn);
+  dbus_clear_connection (&third_conn);
 }
 
 static void
@@ -827,8 +822,8 @@ test_max_replies_per_connection (Fixture *f,
       if (!dbus_connection_send (f->right_conn, reply, NULL))
         g_error ("OOM");
 
-      dbus_message_unref (reply);
-      dbus_message_unref (m);
+      dbus_clear_message (&m);
+      dbus_clear_message (&reply);
     }
 
   /* Wait for all 5 replies to come in. */
@@ -844,7 +839,7 @@ test_max_replies_per_connection (Fixture *f,
       if (dbus_set_error_from_message (&error, m))
         g_error ("Unexpected error: %s: %s", error.name, error.message);
 
-      dbus_message_unref (m);
+      dbus_clear_message (&m);
     }
 
   /* The last two failed. */
@@ -858,7 +853,7 @@ test_max_replies_per_connection (Fixture *f,
 
       g_assert_cmpstr (error.name, ==, DBUS_ERROR_LIMITS_EXCEEDED);
       dbus_error_free (&error);
-      dbus_message_unref (m);
+      dbus_clear_message (&m);
     }
 
   g_assert_cmpuint (g_queue_get_length (&received), ==, 0);
@@ -1048,6 +1043,147 @@ test_pending_fd_timeout (Fixture *f,
   g_object_unref (socket);
 }
 
+typedef struct
+{
+  const gchar *path;
+  guint n_fds;
+  gboolean should_work;
+} CountFdsVector;
+
+static const CountFdsVector count_fds_vectors[] =
+{
+  /* Deny sending if number of fds <= 2 */
+  { "/test/DenySendMax2", 1, FALSE },
+  { "/test/DenySendMax2", 2, FALSE },
+  { "/test/DenySendMax2", 3, TRUE },
+  { "/test/DenySendMax2", 4, TRUE },
+
+  /* Deny receiving if number of fds <= 3 */
+  { "/test/DenyReceiveMax3", 2, FALSE },
+  { "/test/DenyReceiveMax3", 3, FALSE },
+  { "/test/DenyReceiveMax3", 4, TRUE },
+  { "/test/DenyReceiveMax3", 5, TRUE },
+
+  /* Deny sending if number of fds >= 4 */
+  { "/test/DenySendMin4", 2, TRUE },
+  { "/test/DenySendMin4", 3, TRUE },
+  { "/test/DenySendMin4", 4, FALSE },
+  { "/test/DenySendMin4", 5, FALSE },
+
+  /* Deny receiving if number of fds >= 5 */
+  { "/test/DenyReceiveMin5", 3, TRUE },
+  { "/test/DenyReceiveMin5", 4, TRUE },
+  { "/test/DenyReceiveMin5", 5, FALSE },
+  { "/test/DenyReceiveMin5", 6, FALSE },
+};
+
+static void
+test_count_fds (Fixture *f,
+    gconstpointer context)
+{
+  GQueue received = G_QUEUE_INIT;
+  DBusMessage *m;
+  DBusPendingCall *pc;
+  guint i;
+  DBusError error = DBUS_ERROR_INIT;
+  const int stdin_fd = 0;
+
+  if (f->skip)
+    return;
+
+  add_hold_filter (f);
+
+  for (i = 0; i < G_N_ELEMENTS (count_fds_vectors); i++)
+    {
+      const CountFdsVector *vector = &count_fds_vectors[i];
+      guint j;
+
+      m = dbus_message_new_method_call (
+          dbus_bus_get_unique_name (f->right_conn), vector->path,
+          "com.example", "Spam");
+
+      if (m == NULL)
+        g_error ("OOM");
+
+      for (j = 0; j < vector->n_fds; j++)
+        {
+          if (!dbus_message_append_args (m,
+                                         DBUS_TYPE_UNIX_FD, &stdin_fd,
+                                         DBUS_TYPE_INVALID))
+            g_error ("OOM");
+        }
+
+      if (!dbus_connection_send_with_reply (f->left_conn, m, &pc,
+                                            DBUS_TIMEOUT_INFINITE) ||
+          pc == NULL)
+        g_error ("OOM");
+
+      if (dbus_pending_call_get_completed (pc))
+        pc_enqueue (pc, &received);
+      else if (!dbus_pending_call_set_notify (pc, pc_enqueue, &received,
+            NULL))
+        g_error ("OOM");
+
+      dbus_pending_call_unref (pc);
+      dbus_message_unref (m);
+
+      if (vector->should_work)
+        {
+          DBusMessage *reply;
+
+          while (g_queue_get_length (&f->held_messages) < 1)
+            test_main_context_iterate (f->ctx, TRUE);
+
+          g_assert_cmpint (g_queue_get_length (&f->held_messages), ==, 1);
+
+          m = g_queue_pop_head (&f->held_messages);
+
+          g_assert_cmpint (g_queue_get_length (&f->held_messages), ==, 0);
+
+          reply = dbus_message_new_method_return (m);
+
+          if (reply == NULL)
+            g_error ("OOM");
+
+          if (!dbus_connection_send (f->right_conn, reply, NULL))
+            g_error ("OOM");
+
+          dbus_message_unref (reply);
+          dbus_message_unref (m);
+        }
+
+      while (g_queue_get_length (&received) < 1)
+        test_main_context_iterate (f->ctx, TRUE);
+
+      g_assert_cmpint (g_queue_get_length (&received), ==, 1);
+      m = g_queue_pop_head (&received);
+      g_assert (m != NULL);
+      g_assert_cmpint (g_queue_get_length (&received), ==, 0);
+
+      if (vector->should_work)
+        {
+          if (dbus_set_error_from_message (&error, m))
+            g_error ("Unexpected error: %s: %s", error.name, error.message);
+
+          g_test_message ("Sending %u fds to %s was not denied, as expected",
+                          vector->n_fds, vector->path);
+        }
+      else if (!dbus_set_error_from_message (&error, m))
+        {
+          g_error ("Unexpected success");
+        }
+      else
+        {
+          g_assert_cmpstr (error.name, ==, DBUS_ERROR_ACCESS_DENIED);
+          dbus_error_free (&error);
+          g_test_message ("Sending %u fds to %s was denied, as expected",
+                          vector->n_fds, vector->path);
+        }
+
+      dbus_message_unref (m);
+    }
+}
+
 #endif
 
 static void
@@ -1093,8 +1229,7 @@ test_peer_get_machine_id (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1114,8 +1249,8 @@ test_peer_get_machine_id (Fixture *f,
   g_assert_nonnull (what_daemon_thinks);
   g_assert_cmpuint (strlen (what_daemon_thinks), ==, 32);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
   dbus_free (what_i_think);
 }
 
@@ -1140,8 +1275,7 @@ test_peer_ping (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1155,8 +1289,8 @@ test_peer_ping (Fixture *f,
   if (!dbus_message_get_args (m, &error, DBUS_TYPE_INVALID))
     g_error ("%s: %s", error.name, error.message);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1183,8 +1317,7 @@ test_get_invalid_path (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1202,8 +1335,8 @@ test_get_invalid_path (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1230,8 +1363,7 @@ test_get_invalid_iface (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1248,8 +1380,8 @@ test_get_invalid_iface (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1276,8 +1408,7 @@ test_get_invalid (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1294,8 +1425,8 @@ test_get_invalid (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_PROPERTY);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1320,8 +1451,7 @@ test_get_all_invalid_iface (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1338,8 +1468,8 @@ test_get_all_invalid_iface (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1364,8 +1494,7 @@ test_get_all_invalid_path (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1383,8 +1512,8 @@ test_get_all_invalid_path (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1422,8 +1551,7 @@ test_set_invalid_iface (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1440,8 +1568,8 @@ test_set_invalid_iface (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1479,8 +1607,7 @@ test_set_invalid_path (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1497,8 +1624,8 @@ test_set_invalid_path (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_INTERFACE);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1536,8 +1663,7 @@ test_set_invalid (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1554,8 +1680,8 @@ test_set_invalid (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_UNKNOWN_PROPERTY);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1593,8 +1719,7 @@ test_set (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1611,8 +1736,8 @@ test_set (Fixture *f,
   g_assert_cmpstr (error.name, ==, DBUS_ERROR_PROPERTY_READ_ONLY);
   dbus_error_free (&error);
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1678,8 +1803,7 @@ test_features (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1702,8 +1826,8 @@ test_features (Fixture *f,
   if (dbus_message_iter_next (&args_iter))
     g_error ("Reply has too many arguments");
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1784,8 +1908,7 @@ test_interfaces (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1808,8 +1931,8 @@ test_interfaces (Fixture *f,
   if (dbus_message_iter_next (&args_iter))
     g_error ("Reply has too many arguments");
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1839,8 +1962,7 @@ test_get_all (Fixture *f,
       pc == NULL)
     g_error ("OOM");
 
-  dbus_message_unref (m);
-  m = NULL;
+  dbus_clear_message (&m);
 
   if (dbus_pending_call_get_completed (pc))
     test_pending_call_store_reply (pc, &m);
@@ -1891,8 +2013,8 @@ test_get_all (Fixture *f,
   if (dbus_message_iter_next (&args_iter))
     g_error ("Reply has too many arguments");
 
-  dbus_message_unref (m);
-  dbus_pending_call_unref (pc);
+  dbus_clear_message (&m);
+  dbus_clear_pending_call (&pc);
 }
 
 static void
@@ -1903,11 +2025,7 @@ teardown (Fixture *f,
   g_clear_error (&f->ge);
 
   if (f->left_conn != NULL)
-    {
-      dbus_connection_close (f->left_conn);
-      dbus_connection_unref (f->left_conn);
-      f->left_conn = NULL;
-    }
+    dbus_connection_close (f->left_conn);
 
   if (f->right_conn != NULL)
     {
@@ -1927,10 +2045,11 @@ teardown (Fixture *f,
       g_queue_clear (&f->held_messages);
 
       dbus_connection_close (f->right_conn);
-      dbus_connection_unref (f->right_conn);
-      f->right_conn = NULL;
     }
 
+  dbus_clear_connection (&f->left_conn);
+  dbus_clear_connection (&f->right_conn);
+
   if (f->daemon_pid != 0)
     {
       test_kill_pid (f->daemon_pid);
@@ -2010,6 +2129,11 @@ static Config pending_fd_timeout_config = {
     NULL, 1, "valid-config-files/pending-fd-timeout.conf",
     SPECIFY_ADDRESS
 };
+
+static Config count_fds_config = {
+    NULL, 1, "valid-config-files/count-fds.conf",
+    SPECIFY_ADDRESS
+};
 #endif
 
 int
@@ -2076,6 +2200,8 @@ main (int argc,
   g_test_add ("/limits/pending-fd-timeout", Fixture,
       &pending_fd_timeout_config,
       setup, test_pending_fd_timeout, teardown);
+  g_test_add ("/policy/count-fds", Fixture, &count_fds_config,
+      setup, test_count_fds, teardown);
 #endif
 
 #ifdef DBUS_UNIX
