@@ -50,14 +50,6 @@
 
 #include "test-utils-glib.h"
 
-#ifdef DBUS_ENABLE_EMBEDDED_TESTS
-#include <dbus/dbus-message-internal.h>
-#else
-typedef struct _DBusInitialFDs DBusInitialFDs;
-#define _dbus_check_fdleaks_enter() NULL
-#define _dbus_check_fdleaks_leave(fds) do {} while (0)
-#endif
-
 /* Arbitrary; included here to avoid relying on the default */
 #define MAX_MESSAGE_UNIX_FDS 20
 /* This test won't work on Linux unless this is true. */
@@ -99,7 +91,6 @@ typedef struct {
     GQueue messages;
 
     int fd_before;
-    DBusInitialFDs *initial_fds;
 } Fixture;
 
 static void oom (const gchar *doing) G_GNUC_NORETURN;
@@ -181,8 +172,6 @@ test_connect (Fixture *f,
 {
   char *address;
 
-  f->initial_fds = _dbus_check_fdleaks_enter ();
-
   g_assert (f->left_server_conn == NULL);
   g_assert (f->right_server_conn == NULL);
 
@@ -370,7 +359,9 @@ test_relay (Fixture *f,
   g_assert_cmpstr (dbus_message_get_path (incoming), ==, "/com/example/Hello");
   g_assert_cmpuint (dbus_message_get_serial (incoming), ==, serial);
 
-  if (!dbus_message_get_args (incoming,
+  if (dbus_set_error_from_message (&f->e, incoming))
+    g_error ("%s: %s", f->e.name, f->e.message);
+  else if (!dbus_message_get_args (incoming,
         &f->e,
         DBUS_TYPE_UNIX_FD, &fd_after,
         DBUS_TYPE_INVALID))
@@ -846,9 +837,6 @@ teardown (Fixture *f,
   if (f->fd_before >= 0 && close (f->fd_before) < 0)
     g_error ("%s", g_strerror (errno));
 #endif
-
-  if (f->initial_fds != NULL)
-    _dbus_check_fdleaks_leave (f->initial_fds);
 }
 
 int
