@@ -7,7 +7,7 @@ try:
     import dbus.mainloop.glib
     from gi.repository import GObject
 except:
-    print "Failed import, aborting test"
+    print("Failed import, aborting test")
     sys.exit(0)
 
 dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
@@ -16,7 +16,7 @@ loop = GObject.MainLoop()
 exitcode = 0
 
 def handle_noreceipt():
-    print "Failed to get signal"
+    print("Failed to get signal")
     global exitcode
     exitcode = 1
     loop.quit()
@@ -26,7 +26,7 @@ GObject.timeout_add(7000, handle_noreceipt)
 bus = dbus.SessionBus()
 
 def sighandler(*args, **kwargs):
-    print "got signal"
+    print("got signal")
     loop.quit()   
 
 bus.add_signal_receiver(sighandler, dbus_interface='org.freedesktop.TestSuite', signal_name='Foo')
