@@ -34,7 +34,9 @@
 #include <string.h>
 #include <sys/types.h>
 
-#if defined(HAVE_APPARMOR) && defined(DBUS_TEST_APPARMOR_ACTIVATION)
+#include <glib/gstdio.h>
+
+#if defined(HAVE_APPARMOR_2_10) && defined(DBUS_TEST_APPARMOR_ACTIVATION)
 #include <sys/apparmor.h>
 #endif
 
@@ -62,8 +64,23 @@ typedef struct {
     const char *activated_name;
     DBusMessage *activated_message;
     dbus_bool_t activated_filter_added;
+
+    gchar *transient_service_file;
+    gchar *tmp_runtime_dir;
 } Fixture;
 
+typedef enum
+{
+  FLAG_EARLY_TRANSIENT_SERVICE = (1 << 0),
+  FLAG_NONE = 0
+} Flags;
+
+typedef struct
+{
+  const gchar *bus_name;
+  Flags flags;
+} Config;
+
 /* this is a macro so it gets the right line number */
 #define assert_signal(m, \
     sender, path, iface, member, signature, \
@@ -178,6 +195,10 @@ activated_filter (DBusConnection *connection,
   g_assert (f->activated_message == NULL);
   f->activated_message = dbus_message_ref (message);
 
+  /* Test code is expected to reply to method calls itself */
+  if (dbus_message_get_type (message) == DBUS_MESSAGE_TYPE_METHOD_CALL)
+    return DBUS_HANDLER_RESULT_HANDLED;
+
   return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
 }
 
@@ -203,19 +224,65 @@ caller_filter (DBusConnection *connection,
 }
 
 static void
-setup (Fixture *f,
-    gconstpointer context G_GNUC_UNUSED)
+fixture_create_transient_service (Fixture *f,
+                                  const gchar *name)
 {
-#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && !defined(HAVE_APPARMOR)
+  gchar *service;
+  gchar *content;
+  gboolean ok;
 
-  g_test_skip ("AppArmor support not compiled");
+  service = g_strdup_printf ("%s.service", name);
+  f->transient_service_file = g_build_filename (f->tmp_runtime_dir, "dbus-1",
+      "services", service, NULL);
+  g_free (service);
+
+  content = g_strdup_printf (
+      "[D-BUS Service]\n"
+      "Name=%s\n"
+      "Exec=/bin/false %s\n"
+      "SystemdService=dbus-%s.service\n", name, name, name);
+  ok = g_file_set_contents (f->transient_service_file, content, -1, &f->ge);
+  g_assert_no_error (f->ge);
+  g_assert (ok);
+  g_free (content);
+}
+
+static void
+setup (Fixture *f,
+    gconstpointer context)
+{
+  const Config *config = context;
+#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && defined(HAVE_APPARMOR_2_10)
+  aa_features *features;
+#endif
+
+  f->ge = NULL;
+  dbus_error_init (&f->e);
+
+  f->tmp_runtime_dir = g_dir_make_tmp ("dbus-daemon-test.XXXXXX", &f->ge);
+  g_assert_no_error (f->ge);
+
+  if (config != NULL && (config->flags & FLAG_EARLY_TRANSIENT_SERVICE) != 0)
+    {
+      gchar *dbus1 = g_build_filename (f->tmp_runtime_dir, "dbus-1", NULL);
+      gchar *services = g_build_filename (dbus1, "services", NULL);
+
+      /* We just created it so the directories shouldn't exist yet */
+      test_mkdir (dbus1, 0700);
+      test_mkdir (services, 0700);
+      fixture_create_transient_service (f, config->bus_name);
+      g_free (dbus1);
+      g_free (services);
+    }
+
+#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && !defined(HAVE_APPARMOR_2_10)
+
+  g_test_skip ("AppArmor support not compiled or AppArmor 2.10 unavailable");
   return;
 
 #else
 
 #if defined(DBUS_TEST_APPARMOR_ACTIVATION)
-  aa_features *features;
-
   if (!aa_is_enabled ())
     {
       g_test_message ("aa_is_enabled() -> %s", g_strerror (errno));
@@ -242,12 +309,9 @@ setup (Fixture *f,
 
   f->ctx = test_main_context_get ();
 
-  f->ge = NULL;
-  dbus_error_init (&f->e);
-
   f->address = test_get_dbus_daemon (
       "valid-config-files/systemd-activation.conf",
-      TEST_USER_ME, &f->daemon_pid);
+      TEST_USER_ME, f->tmp_runtime_dir, &f->daemon_pid);
 
   if (f->address == NULL)
     return;
@@ -624,7 +688,10 @@ test_deny_send (Fixture *f,
     gconstpointer context)
 {
   DBusMessage *m;
-  const char *bus_name = context;
+  const Config *config = context;
+
+  g_assert (config != NULL);
+  g_assert (config->bus_name != NULL);
 
   if (f->address == NULL)
     return;
@@ -635,7 +702,7 @@ test_deny_send (Fixture *f,
   f->caller_filter_added = TRUE;
 
   /* The sender sends a message to an activatable service. */
-  m = dbus_message_new_method_call (bus_name, "/foo",
+  m = dbus_message_new_method_call (config->bus_name, "/foo",
       "com.example.bar", "Call");
   if (m == NULL)
     g_error ("OOM");
@@ -673,7 +740,10 @@ test_deny_receive (Fixture *f,
     gconstpointer context)
 {
   DBusMessage *m;
-  const char *bus_name = context;
+  const Config *config = context;
+
+  g_assert (config != NULL);
+  g_assert (config->bus_name != NULL);
 
   if (f->address == NULL)
     return;
@@ -686,7 +756,8 @@ test_deny_receive (Fixture *f,
   /* The sender sends a message to an activatable service.
    * We set the interface name equal to the bus name to make it
    * easier to write the necessary policy rules. */
-  m = dbus_message_new_method_call (bus_name, "/foo", bus_name, "Call");
+  m = dbus_message_new_method_call (config->bus_name, "/foo",
+                                    config->bus_name, "Call");
   if (m == NULL)
     g_error ("OOM");
 
@@ -714,11 +785,11 @@ test_deny_receive (Fixture *f,
 
   /* systemd starts the activatable service. */
 
-#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && defined(HAVE_APPARMOR)
+#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && defined(HAVE_APPARMOR_2_10)
   /* The use of 42 here is arbitrary, see setup(). */
-  if (aa_change_hat (bus_name, 42) != 0)
+  if (aa_change_hat (config->bus_name, 42) != 0)
     g_error ("Unable to change profile to ...//^%s: %s",
-             bus_name, g_strerror (errno));
+             config->bus_name, g_strerror (errno));
 #endif
 
   f->activated = test_connect_to_bus (f->ctx, f->address);
@@ -727,9 +798,9 @@ test_deny_receive (Fixture *f,
     g_error ("OOM");
   f->activated_filter_added = TRUE;
   f->activated_name = dbus_bus_get_unique_name (f->activated);
-  take_well_known_name (f, f->activated, bus_name);
+  take_well_known_name (f, f->activated, config->bus_name);
 
-#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && defined(HAVE_APPARMOR)
+#if defined(DBUS_TEST_APPARMOR_ACTIVATION) && defined(HAVE_APPARMOR_2_10)
   if (aa_change_hat (NULL, 42) != 0)
     g_error ("Unable to change back to initial profile: %s",
              g_strerror (errno));
@@ -764,6 +835,172 @@ test_deny_receive (Fixture *f,
   g_assert (f->activated_message == NULL);
 }
 
+/*
+ * Test that we can set up transient services.
+ *
+ * If (flags & FLAG_EARLY_TRANSIENT_SERVICE), we assert that a service that
+ * was deployed before starting systemd (in setup()) is available.
+ *
+ * Otherwise, we assert that a service that is deployed while dbus-daemon
+ * is already running becomes available after reloading the dbus-daemon
+ * configuration.
+ */
+static void
+test_transient_services (Fixture *f,
+    gconstpointer context)
+{
+  const Config *config = context;
+  DBusMessage *m = NULL;
+  DBusMessage *send_reply = NULL;
+  DBusMessage *reply = NULL;
+  DBusPendingCall *pc;
+
+  g_assert (config != NULL);
+  g_assert (config->bus_name != NULL);
+
+  if (f->address == NULL)
+    return;
+
+  /* Connect the fake systemd to the bus. */
+  f->systemd = test_connect_to_bus (f->ctx, f->address);
+  if (!dbus_connection_add_filter (f->systemd, systemd_filter, f, NULL))
+    g_error ("OOM");
+  f->systemd_filter_added = TRUE;
+  f->systemd_name = dbus_bus_get_unique_name (f->systemd);
+  take_well_known_name (f, f->systemd, "org.freedesktop.systemd1");
+
+  if ((config->flags & FLAG_EARLY_TRANSIENT_SERVICE) == 0)
+    {
+      /* Try to activate a service that isn't there. */
+      m = dbus_message_new_method_call (config->bus_name,
+                                        "/foo", "com.example.bar", "Activate");
+
+      if (m == NULL ||
+          !dbus_connection_send_with_reply (f->caller, m, &pc,
+            DBUS_TIMEOUT_USE_DEFAULT) || pc == NULL)
+        g_error ("OOM");
+
+      dbus_message_unref (m);
+      m = NULL;
+
+      /* It fails. */
+
+      if (dbus_pending_call_get_completed (pc))
+        test_pending_call_store_reply (pc, &m);
+      else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+            &m, NULL))
+        g_error ("OOM");
+
+      while (m == NULL)
+        test_main_context_iterate (f->ctx, TRUE);
+
+      assert_error_reply (m, DBUS_SERVICE_DBUS, f->caller_name,
+          DBUS_ERROR_SERVICE_UNKNOWN);
+
+      dbus_message_unref (m);
+      m = NULL;
+
+      /* Now generate a transient D-Bus service file for it. The directory
+       * should have been created during dbus-daemon startup, so we don't have to
+       * recreate it. */
+      fixture_create_transient_service (f, config->bus_name);
+
+      /* To guarantee that the transient service has been picked up, we have
+       * to reload. */
+      m = dbus_message_new_method_call (DBUS_SERVICE_DBUS, DBUS_PATH_DBUS,
+                                        DBUS_INTERFACE_DBUS, "ReloadConfig");
+
+      if (m == NULL ||
+          !dbus_connection_send_with_reply (f->caller, m, &pc,
+            DBUS_TIMEOUT_USE_DEFAULT) || pc == NULL)
+        g_error ("OOM");
+
+      dbus_message_unref (m);
+      m = NULL;
+
+      if (dbus_pending_call_get_completed (pc))
+        test_pending_call_store_reply (pc, &m);
+      else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+            &m, NULL))
+        g_error ("OOM");
+
+      while (m == NULL)
+        test_main_context_iterate (f->ctx, TRUE);
+
+      assert_method_reply (m, DBUS_SERVICE_DBUS, f->caller_name, "");
+      dbus_message_unref (m);
+      m = NULL;
+    }
+
+  /* The service is present now. */
+  m = dbus_message_new_method_call (config->bus_name,
+                                    "/foo", "com.example.bar", "Activate");
+
+  if (m == NULL ||
+      !dbus_connection_send_with_reply (f->caller, m, &pc,
+        DBUS_TIMEOUT_USE_DEFAULT) || pc == NULL)
+    g_error ("OOM");
+
+  dbus_message_unref (m);
+  m = NULL;
+
+  if (dbus_pending_call_get_completed (pc))
+    test_pending_call_store_reply (pc, &reply);
+  else if (!dbus_pending_call_set_notify (pc, test_pending_call_store_reply,
+        &reply, NULL))
+    g_error ("OOM");
+
+  /* The mock systemd is told to start the service. */
+  while (f->systemd_message == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  m = f->systemd_message;
+  f->systemd_message = NULL;
+  assert_signal (m, DBUS_SERVICE_DBUS, DBUS_PATH_DBUS,
+      "org.freedesktop.systemd1.Activator", "ActivationRequest", "s",
+      "org.freedesktop.systemd1");
+  dbus_message_unref (m);
+  m = NULL;
+
+  /* The activatable service connects and gets its name. */
+  f->activated = test_connect_to_bus (f->ctx, f->address);
+  if (!dbus_connection_add_filter (f->activated, activated_filter,
+        f, NULL))
+    g_error ("OOM");
+  f->activated_filter_added = TRUE;
+  f->activated_name = dbus_bus_get_unique_name (f->activated);
+  take_well_known_name (f, f->activated, config->bus_name);
+
+  /* The message is delivered to the activatable service. */
+  while (f->activated_message == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  m = f->activated_message;
+  f->activated_message = NULL;
+  assert_method_call (m, f->caller_name, config->bus_name, "/foo",
+      "com.example.bar", "Activate", "");
+
+  /* The activatable service sends back a reply. */
+  send_reply = dbus_message_new_method_return (m);
+
+  if (send_reply == NULL ||
+      !dbus_connection_send (f->activated, send_reply, NULL))
+    g_error ("OOM");
+
+  dbus_message_unref (send_reply);
+  send_reply = NULL;
+  dbus_message_unref (m);
+  m = NULL;
+
+  /* The caller receives the reply. */
+  while (reply == NULL)
+    test_main_context_iterate (f->ctx, TRUE);
+
+  assert_method_reply (reply, f->activated_name, f->caller_name, "");
+  dbus_message_unref (reply);
+  reply = NULL;
+}
+
 static void
 teardown (Fixture *f,
     gconstpointer context G_GNUC_UNUSED)
@@ -801,43 +1038,105 @@ teardown (Fixture *f,
       f->activated = NULL;
     }
 
-  test_kill_pid (f->daemon_pid);
-  g_spawn_close_pid (f->daemon_pid);
-  test_main_context_unref (f->ctx);
+  if (f->daemon_pid != 0)
+    {
+      test_kill_pid (f->daemon_pid);
+      g_spawn_close_pid (f->daemon_pid);
+    }
+
+  if (f->ctx != NULL)
+    test_main_context_unref (f->ctx);
+
   g_free (f->address);
+
+  if (f->transient_service_file != NULL)
+    {
+      test_remove_if_exists (f->transient_service_file);
+      g_free (f->transient_service_file);
+    }
+
+  if (f->tmp_runtime_dir != NULL)
+    {
+      gchar *dbus1 = g_build_filename (f->tmp_runtime_dir, "dbus-1", NULL);
+      gchar *services = g_build_filename (dbus1, "services", NULL);
+
+      test_rmdir_if_exists (services);
+      test_rmdir_if_exists (dbus1);
+      test_rmdir_if_exists (f->tmp_runtime_dir);
+
+      g_free (f->tmp_runtime_dir);
+      g_free (dbus1);
+      g_free (services);
+    }
 }
 
+static const Config deny_send_tests[] =
+{
+#if defined(DBUS_TEST_APPARMOR_ACTIVATION)
+    { "com.example.SendDeniedByAppArmorLabel" },
+    { "com.example.SendDeniedByNonexistentAppArmorLabel" },
+    { "com.example.SendDeniedByAppArmorName" },
+#endif
+    { "com.example.SendDenied" }
+};
+
+static const Config deny_receive_tests[] =
+{
+#if defined(DBUS_TEST_APPARMOR_ACTIVATION)
+    { "com.example.ReceiveDeniedByAppArmorLabel" },
+#endif
+    { "com.example.ReceiveDenied" }
+};
+
+static const Config transient_service_later =
+{
+  "com.example.TransientActivatable1",
+  FLAG_NONE
+};
+
+static const Config transient_service_in_advance =
+{
+  "com.example.TransientActivatable1",
+  FLAG_EARLY_TRANSIENT_SERVICE
+};
+
 int
 main (int argc,
     char **argv)
 {
+  gsize i;
+
   test_init (&argc, &argv);
 
   g_test_add ("/sd-activation/activation", Fixture, NULL,
       setup, test_activation, teardown);
   g_test_add ("/sd-activation/uae", Fixture, NULL,
       setup, test_uae, teardown);
-  g_test_add ("/sd-activation/deny-send", Fixture,
-      "com.example.SendDenied",
-      setup, test_deny_send, teardown);
-  g_test_add ("/sd-activation/deny-receive", Fixture,
-      "com.example.ReceiveDenied",
-      setup, test_deny_receive, teardown);
 
-#if defined(DBUS_TEST_APPARMOR_ACTIVATION)
-  g_test_add ("/sd-activation/apparmor/deny-send/by-label", Fixture,
-      "com.example.SendDeniedByAppArmorLabel",
-      setup, test_deny_send, teardown);
-  g_test_add ("/sd-activation/apparmor/deny-send/by-nonexistent-label", Fixture,
-      "com.example.SendDeniedByNonexistentAppArmorLabel",
-      setup, test_deny_send, teardown);
-  g_test_add ("/sd-activation/apparmor/deny-send/by-name", Fixture,
-      "com.example.SendDeniedByAppArmorName",
-      setup, test_deny_send, teardown);
-  g_test_add ("/sd-activation/apparmor/deny-receive/by-label", Fixture,
-      "com.example.ReceiveDeniedByAppArmorLabel",
-      setup, test_deny_receive, teardown);
-#endif
+  for (i = 0; i < G_N_ELEMENTS (deny_send_tests); i++)
+    {
+      gchar *name = g_strdup_printf ("/sd-activation/deny-send/%s",
+                                     deny_send_tests[i].bus_name);
+
+      g_test_add (name, Fixture, &deny_send_tests[i],
+                  setup, test_deny_send, teardown);
+      g_free (name);
+    }
+
+  for (i = 0; i < G_N_ELEMENTS (deny_receive_tests); i++)
+    {
+      gchar *name = g_strdup_printf ("/sd-activation/deny-receive/%s",
+                                     deny_receive_tests[i].bus_name);
+
+      g_test_add (name, Fixture, &deny_receive_tests[i],
+                  setup, test_deny_receive, teardown);
+      g_free (name);
+    }
+
+  g_test_add ("/sd-activation/transient-services/later", Fixture,
+      &transient_service_later, setup, test_transient_services, teardown);
+  g_test_add ("/sd-activation/transient-services/in-advance", Fixture,
+      &transient_service_in_advance, setup, test_transient_services, teardown);
 
   return g_test_run ();
 }
