@@ -459,7 +459,9 @@ test_kill_pid (GPid pid)
 static gboolean
 time_out (gpointer data)
 {
-  g_error ("timed out");
+  puts ("Bail out! Test timed out (GLib main loop timeout callback reached)");
+  fflush (stdout);
+  abort ();
   return FALSE;
 }
 
@@ -469,28 +471,40 @@ static void wrap_abort (int signal) _DBUS_GNUC_NORETURN;
 static void
 wrap_abort (int signal)
 {
+  /* We might be halfway through writing out something else, so force this
+   * onto its own line */
+  const char message [] = "\nBail out! Test timed out (SIGALRM received)\n";
+
+  if (write (STDOUT_FILENO, message, sizeof (message) - 1) <
+      (ssize_t) sizeof (message) - 1)
+    {
+      /* ignore short write - what would we do about it? */
+    }
+
   abort ();
 }
 #endif
 
-void
-test_init (int *argcp, char ***argvp)
+static void
+set_timeout (void)
 {
-  g_test_init (argcp, argvp, NULL);
-  g_test_bug_base ("https://bugs.freedesktop.org/show_bug.cgi?id=");
+  static guint timeout = 0;
 
   /* Prevent tests from hanging forever. This is intended to be long enough
    * that any reasonable regression test on any reasonable hardware would
    * have finished. */
 #define TIMEOUT 60
 
-  g_timeout_add_seconds (TIMEOUT, time_out, NULL);
+  if (timeout != 0)
+    g_source_remove (timeout);
+
+  timeout = g_timeout_add_seconds (TIMEOUT, time_out, NULL);
 #ifdef G_OS_UNIX
   /* The GLib main loop might not be running (we don't use it in every
    * test). Die with SIGALRM shortly after if necessary. */
   alarm (TIMEOUT + 10);
 
-  /* Get a core dump from the SIGALRM. */
+  /* Get a log message and a core dump from the SIGALRM. */
     {
       struct sigaction act = { };
 
@@ -501,6 +515,35 @@ test_init (int *argcp, char ***argvp)
 #endif
 }
 
+void
+test_init (int *argcp, char ***argvp)
+{
+  g_test_init (argcp, argvp, NULL);
+  g_test_bug_base ("https://bugs.freedesktop.org/show_bug.cgi?id=");
+  set_timeout ();
+}
+
+static void
+report_and_destroy (gpointer p)
+{
+  GTimer *timer = p;
+
+  g_test_message ("Time since timeout reset %p: %.3f seconds",
+      timer, g_timer_elapsed (timer, NULL));
+  g_timer_destroy (timer);
+}
+
+void
+test_timeout_reset (void)
+{
+  GTimer *timer = g_timer_new ();
+
+  g_test_message ("Resetting test timeout (reference: %p)", timer);
+  set_timeout ();
+
+  g_test_queue_destroy (report_and_destroy, timer);
+}
+
 void
 test_progress (char symbol)
 {
