@@ -37,6 +37,25 @@
 #include <stdlib.h>
 #include "config.h"
 
+#include "tool-common.h"
+
+#ifdef HAVE_INTTYPES_H
+#include <inttypes.h>
+#endif
+
+#if defined(DBUS_WIN)
+#if !defined(PRId64)
+#define PRId64 "I64d"
+#endif
+#if !defined(PRIu64)
+#define PRIu64 "I64u"
+#endif
+#endif
+
+#ifndef HAVE_SOCKLEN_T
+#define socklen_t int
+#endif
+
 static const char*
 type_to_name (int message_type)
 {
@@ -65,7 +84,9 @@ indent (int depth)
 }
 
 static void
-print_hex (unsigned char *bytes, unsigned int len, int depth)
+print_hex (const unsigned char *bytes,
+           unsigned int len,
+           int depth)
 {
   unsigned int i, columns;
 
@@ -110,53 +131,51 @@ print_hex (unsigned char *bytes, unsigned int len, int depth)
 static void
 print_ay (DBusMessageIter *iter, int depth)
 {
-  /* Not using DBusString because it's not public API. It's 2009, and I'm
-   * manually growing a string chunk by chunk.
-   */
-  unsigned char *bytes = malloc (DEFAULT_SIZE + 1);
-  unsigned int len = 0;
-  unsigned int max = DEFAULT_SIZE;
+  /* True if every byte in the bytestring (so far) is printable
+   * ASCII, with one exception: the last byte is also allowed to be \0. */
   dbus_bool_t all_ascii = TRUE;
-  int current_type;
+  const unsigned char *bytes;
+  int len;
+  int i;
 
-  while ((current_type = dbus_message_iter_get_arg_type (iter))
-          != DBUS_TYPE_INVALID)
+  dbus_message_iter_get_fixed_array (iter, &bytes, &len);
+
+  for (i = 0; i < len; i++)
     {
-      unsigned char val;
-
-      dbus_message_iter_get_basic (iter, &val);
-      bytes[len] = val;
-      len++;
-
-      if (val < 32 || val > 126)
-        all_ascii = FALSE;
-
-      if (len == max)
+      if ((bytes[i] < 32 || bytes[i] > 126) &&
+          (i < len - 1 || bytes[i] != '\0'))
         {
-          max *= 2;
-          bytes = realloc (bytes, max + 1);
+          all_ascii = FALSE;
+          break;
         }
-
-      dbus_message_iter_next (iter);
     }
 
-  if (all_ascii)
+  if (all_ascii && len > 0 && bytes[len - 1] == '\0')
     {
-      bytes[len] = '\0';
-      printf ("array of bytes \"%s\"\n", bytes);
+      printf ("array of bytes \"%s\" + \\0\n", bytes);
+    }
+  else if (all_ascii)
+    {
+      unsigned char *copy = dbus_malloc (len + 1);
+
+      if (copy == NULL)
+        tool_oom ("copying bytestring");
+
+      memcpy (copy, bytes, len);
+      copy[len] = '\0';
+      printf ("array of bytes \"%s\"\n", copy);
+      dbus_free (copy);
     }
   else
     {
       print_hex (bytes, len, depth);
     }
-
-  free (bytes);
 }
 
+#ifdef DBUS_UNIX
 static void
 print_fd (int fd, int depth)
 {
-#ifdef DBUS_UNIX
   int ret;
   struct stat statbuf = {0,};
   union {
@@ -167,10 +186,9 @@ print_fd (int fd, int depth)
       struct sockaddr_in6 ipv6;
   } addr, peer;
   char hostip[INET6_ADDRSTRLEN];
-  int addrlen = sizeof (addr);
-  int peerlen = sizeof (peer);
+  socklen_t addrlen = sizeof (addr);
+  socklen_t peerlen = sizeof (peer);
   int has_peer;
-#endif
 
   /* Don't print the fd number: it is different in every process and since
    * dbus-monitor closes the fd after reading it, the same number would be
@@ -180,7 +198,6 @@ print_fd (int fd, int depth)
   if (fd == -1)
     return;
 
-#ifdef DBUS_UNIX
   ret = fstat (fd, &statbuf);
   if (ret == -1)
     return;
@@ -295,8 +312,8 @@ print_fd (int fd, int depth)
         printf ("unknown (%d)\n", addr.sa.sa_family);
         break;
     }
-#endif
 }
+#endif
 
 static void
 print_iter (DBusMessageIter *iter, dbus_bool_t literal, int depth)
@@ -306,209 +323,202 @@ print_iter (DBusMessageIter *iter, dbus_bool_t literal, int depth)
       int type = dbus_message_iter_get_arg_type (iter);
 
       if (type == DBUS_TYPE_INVALID)
-	break;
+        break;
       
       indent(depth);
 
       switch (type)
-	{
-	case DBUS_TYPE_STRING:
-	  {
-	    char *val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    if (!literal)
-	      printf ("string \"");
-	    printf ("%s", val);
-	    if (!literal)
-	      printf ("\"\n");
-	    break;
-	  }
+        {
+        case DBUS_TYPE_STRING:
+          {
+            char *val;
+            dbus_message_iter_get_basic (iter, &val);
+            if (!literal)
+              printf ("string \"");
+            printf ("%s", val);
+            if (!literal)
+              printf ("\"\n");
+            break;
+          }
 
-	case DBUS_TYPE_SIGNATURE:
-	  {
-	    char *val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    if (!literal)
-	      printf ("signature \"");
-	    printf ("%s", val);
-	    if (!literal)
-	      printf ("\"\n");
-	    break;
-	  }
+        case DBUS_TYPE_SIGNATURE:
+          {
+            char *val;
+            dbus_message_iter_get_basic (iter, &val);
+            if (!literal)
+              printf ("signature \"");
+            printf ("%s", val);
+            if (!literal)
+              printf ("\"\n");
+            break;
+          }
 
-	case DBUS_TYPE_OBJECT_PATH:
-	  {
-	    char *val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    if (!literal)
-	      printf ("object path \"");
-	    printf ("%s", val);
-	    if (!literal)
-	      printf ("\"\n");
-	    break;
-	  }
+        case DBUS_TYPE_OBJECT_PATH:
+          {
+            char *val;
+            dbus_message_iter_get_basic (iter, &val);
+            if (!literal)
+              printf ("object path \"");
+            printf ("%s", val);
+            if (!literal)
+              printf ("\"\n");
+            break;
+          }
 
-	case DBUS_TYPE_INT16:
-	  {
-	    dbus_int16_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("int16 %d\n", val);
-	    break;
-	  }
+        case DBUS_TYPE_INT16:
+          {
+            dbus_int16_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("int16 %d\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_UINT16:
-	  {
-	    dbus_uint16_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("uint16 %u\n", val);
-	    break;
-	  }
+        case DBUS_TYPE_UINT16:
+          {
+            dbus_uint16_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("uint16 %u\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_INT32:
-	  {
-	    dbus_int32_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("int32 %d\n", val);
-	    break;
-	  }
+        case DBUS_TYPE_INT32:
+          {
+            dbus_int32_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("int32 %d\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_UINT32:
-	  {
-	    dbus_uint32_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("uint32 %u\n", val);
-	    break;
-	  }
+        case DBUS_TYPE_UINT32:
+          {
+            dbus_uint32_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("uint32 %u\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_INT64:
-	  {
-	    dbus_int64_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-#ifdef DBUS_INT64_PRINTF_MODIFIER
-        printf ("int64 %" DBUS_INT64_PRINTF_MODIFIER "d\n", val);
-#else
-        printf ("int64 (omitted)\n");
-#endif
-	    break;
-	  }
+        case DBUS_TYPE_INT64:
+          {
+            dbus_int64_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("int64 %" PRId64 "\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_UINT64:
-	  {
-	    dbus_uint64_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-#ifdef DBUS_INT64_PRINTF_MODIFIER
-        printf ("uint64 %" DBUS_INT64_PRINTF_MODIFIER "u\n", val);
-#else
-        printf ("uint64 (omitted)\n");
-#endif
-	    break;
-	  }
+        case DBUS_TYPE_UINT64:
+          {
+            dbus_uint64_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("uint64 %" PRIu64 "\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_DOUBLE:
-	  {
-	    double val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("double %g\n", val);
-	    break;
-	  }
+        case DBUS_TYPE_DOUBLE:
+          {
+            double val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("double %g\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_BYTE:
-	  {
-	    unsigned char val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("byte %d\n", val);
-	    break;
-	  }
+        case DBUS_TYPE_BYTE:
+          {
+            unsigned char val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("byte %d\n", val);
+            break;
+          }
 
-	case DBUS_TYPE_BOOLEAN:
-	  {
-	    dbus_bool_t val;
-	    dbus_message_iter_get_basic (iter, &val);
-	    printf ("boolean %s\n", val ? "true" : "false");
-	    break;
-	  }
+        case DBUS_TYPE_BOOLEAN:
+          {
+            dbus_bool_t val;
+            dbus_message_iter_get_basic (iter, &val);
+            printf ("boolean %s\n", val ? "true" : "false");
+            break;
+          }
 
-	case DBUS_TYPE_VARIANT:
-	  {
-	    DBusMessageIter subiter;
+        case DBUS_TYPE_VARIANT:
+          {
+            DBusMessageIter subiter;
 
-	    dbus_message_iter_recurse (iter, &subiter);
+            dbus_message_iter_recurse (iter, &subiter);
 
-	    printf ("variant ");
-	    print_iter (&subiter, literal, depth+1);
-	    break;
-	  }
-	case DBUS_TYPE_ARRAY:
-	  {
-	    int current_type;
-	    DBusMessageIter subiter;
+            printf ("variant ");
+            print_iter (&subiter, literal, depth+1);
+            break;
+          }
+        case DBUS_TYPE_ARRAY:
+          {
+            int current_type;
+            DBusMessageIter subiter;
 
-	    dbus_message_iter_recurse (iter, &subiter);
+            dbus_message_iter_recurse (iter, &subiter);
 
-	    current_type = dbus_message_iter_get_arg_type (&subiter);
+            current_type = dbus_message_iter_get_arg_type (&subiter);
 
-	    if (current_type == DBUS_TYPE_BYTE)
-	      {
-		print_ay (&subiter, depth);
-		break;
-	      }
+            if (current_type == DBUS_TYPE_BYTE)
+              {
+                print_ay (&subiter, depth);
+                break;
+              }
 
-	    printf("array [\n");
-	    while (current_type != DBUS_TYPE_INVALID)
-	      {
-		print_iter (&subiter, literal, depth+1);
+            printf("array [\n");
+            while (current_type != DBUS_TYPE_INVALID)
+              {
+                print_iter (&subiter, literal, depth+1);
 
-		dbus_message_iter_next (&subiter);
-		current_type = dbus_message_iter_get_arg_type (&subiter);
+                dbus_message_iter_next (&subiter);
+                current_type = dbus_message_iter_get_arg_type (&subiter);
 
-		if (current_type != DBUS_TYPE_INVALID)
-		  printf (",");
-	      }
-	    indent(depth);
-	    printf("]\n");
-	    break;
-	  }
-	case DBUS_TYPE_DICT_ENTRY:
-	  {
-	    DBusMessageIter subiter;
+                if (current_type != DBUS_TYPE_INVALID)
+                  printf (",");
+              }
+            indent(depth);
+            printf("]\n");
+            break;
+          }
+        case DBUS_TYPE_DICT_ENTRY:
+          {
+            DBusMessageIter subiter;
 
-	    dbus_message_iter_recurse (iter, &subiter);
+            dbus_message_iter_recurse (iter, &subiter);
 
-	    printf("dict entry(\n");
-	    print_iter (&subiter, literal, depth+1);
-	    dbus_message_iter_next (&subiter);
-	    print_iter (&subiter, literal, depth+1);
-	    indent(depth);
-	    printf(")\n");
-	    break;
-	  }
-	    
-	case DBUS_TYPE_STRUCT:
-	  {
-	    int current_type;
-	    DBusMessageIter subiter;
+            printf("dict entry(\n");
+            print_iter (&subiter, literal, depth+1);
+            dbus_message_iter_next (&subiter);
+            print_iter (&subiter, literal, depth+1);
+            indent(depth);
+            printf(")\n");
+            break;
+          }
 
-	    dbus_message_iter_recurse (iter, &subiter);
+        case DBUS_TYPE_STRUCT:
+          {
+            int current_type;
+            DBusMessageIter subiter;
 
-	    printf("struct {\n");
-	    while ((current_type = dbus_message_iter_get_arg_type (&subiter)) != DBUS_TYPE_INVALID)
-	      {
-		print_iter (&subiter, literal, depth+1);
-		dbus_message_iter_next (&subiter);
-		if (dbus_message_iter_get_arg_type (&subiter) != DBUS_TYPE_INVALID)
-		  printf (",");
-	      }
-	    indent(depth);
-	    printf("}\n");
-	    break;
-	  }
+            dbus_message_iter_recurse (iter, &subiter);
 
-	case DBUS_TYPE_UNIX_FD:
-	  {
-	    int fd;
-	    dbus_message_iter_get_basic (iter, &fd);
+            printf("struct {\n");
+            while ((current_type = dbus_message_iter_get_arg_type (&subiter)) != DBUS_TYPE_INVALID)
+              {
+                print_iter (&subiter, literal, depth+1);
+                dbus_message_iter_next (&subiter);
+                if (dbus_message_iter_get_arg_type (&subiter) != DBUS_TYPE_INVALID)
+                  printf (",");
+              }
+            indent(depth);
+            printf("}\n");
+            break;
+          }
 
-	    print_fd (fd, depth+1);
+#ifdef DBUS_UNIX
+        case DBUS_TYPE_UNIX_FD:
+          {
+            int fd;
+            dbus_message_iter_get_basic (iter, &fd);
+
+            print_fd (fd, depth+1);
 
             /* dbus_message_iter_get_basic() duplicated the fd, we need to
              * close it after use. The original fd will be closed when the
@@ -516,18 +526,19 @@ print_iter (DBusMessageIter *iter, dbus_bool_t literal, int depth)
              */
             close (fd);
 
-	    break;
-	  }
+            break;
+          }
+#endif
 
-	default:
-	  printf (" (dbus-monitor too dumb to decipher arg type '%c')\n", type);
-	  break;
-	}
+        default:
+          printf (" (dbus-monitor too dumb to decipher arg type '%c')\n", type);
+          break;
+        }
     } while (dbus_message_iter_next (iter));
 }
 
 void
-print_message (DBusMessage *message, dbus_bool_t literal)
+print_message (DBusMessage *message, dbus_bool_t literal, long sec, long usec)
 {
   DBusMessageIter iter;
   const char *sender;
@@ -540,37 +551,48 @@ print_message (DBusMessage *message, dbus_bool_t literal)
   
   if (!literal)
     {
-      printf ("%s sender=%s -> dest=%s",
-	      type_to_name (message_type),
-	      sender ? sender : "(null sender)",
-	      destination ? destination : "(null destination)");
-  
+      if (sec != 0 || usec != 0)
+        {
+          printf ("%s time=%ld.%06ld sender=%s -> destination=%s",
+                  type_to_name (message_type), sec, usec,
+                  sender ? sender : "(null sender)",
+                  destination ? destination : "(null destination)");
+        }
+      else
+        {
+          printf ("%s sender=%s -> destination=%s",
+                  type_to_name (message_type),
+                  sender ? sender : "(null sender)",
+                  destination ? destination : "(null destination)");
+        }
+
       switch (message_type)
-	{
-	case DBUS_MESSAGE_TYPE_METHOD_CALL:
-	case DBUS_MESSAGE_TYPE_SIGNAL:
-	  printf (" serial=%u path=%s; interface=%s; member=%s\n",
+        {
+        case DBUS_MESSAGE_TYPE_METHOD_CALL:
+        case DBUS_MESSAGE_TYPE_SIGNAL:
+          printf (" serial=%u path=%s; interface=%s; member=%s\n",
                   dbus_message_get_serial (message),
-		  dbus_message_get_path (message),
-		  dbus_message_get_interface (message),
-		  dbus_message_get_member (message));
-	  break;
+                  dbus_message_get_path (message),
+                  dbus_message_get_interface (message),
+                  dbus_message_get_member (message));
+          break;
       
-	case DBUS_MESSAGE_TYPE_METHOD_RETURN:
-	  printf (" reply_serial=%u\n",
+        case DBUS_MESSAGE_TYPE_METHOD_RETURN:
+          printf (" serial=%u reply_serial=%u\n",
+                  dbus_message_get_serial (message),
           dbus_message_get_reply_serial (message));
-	  break;
+          break;
 
-	case DBUS_MESSAGE_TYPE_ERROR:
-	  printf (" error_name=%s reply_serial=%u\n",
-		  dbus_message_get_error_name (message),
+        case DBUS_MESSAGE_TYPE_ERROR:
+          printf (" error_name=%s reply_serial=%u\n",
+                  dbus_message_get_error_name (message),
           dbus_message_get_reply_serial (message));
-	  break;
+          break;
 
-	default:
-	  printf ("\n");
-	  break;
-	}
+        default:
+          printf ("\n");
+          break;
+        }
     }
 
   dbus_message_iter_init (message, &iter);
